#!/usr/bin/env python
import os
import sys
import time
import thread
import datetime
import threading
import subprocess
import collections
import commands

DATA_LIST = './.data_list'

data_list = []
for line in open(DATA_LIST):
    line = line.strip()
    if len(line) == 0: continue
    data_list.append(line)

SPEEDUP_LOG_DIR = './log/speedup'
PERFORMANCE_LOG_DIR = './log/performance'
BLOCK_NONBLOCK_LOG_DIR = './log/block_nonblock'
RAND_NONRAND_LOG_DIR = './log/rand_nonrand'
NUM_BLOCK_LOG_DIR = './log/num_block'
NMF_LOG_DIR = './log/nmf'

PARSED_SPEEDUP_LOG_DIR = './log_parsed/speedup'
PARSED_PERFORMANCE_LOG_DIR = './log_parsed/performance'
PARSED_BLOCK_NONBLOCK_LOG_DIR = './log_parsed/block_nonblock'
PARSED_RAND_NONRAND_LOG_DIR = './log_parsed/rand_nonrand'
PARSED_NUM_BLOCK_LOG_DIR = './log_parsed/num_block'
PARSED_NMF_LOG_DIR = './log_parsed/nmf'

methods_list = ['FPSGD', 'DSGD', 'HG', 'CCD']

def os_system(cmd):
    print cmd
    os.system(cmd)

def subp_run(log_tmp, log, cmd):
        print cmd
        f = open(log_tmp, 'w')
        subp = subprocess.Popen(cmd.split(), shell=False, stdout=f)
        subp.wait()
        f.close()
        
        cmd = 'mv {0} {1}'.format(log_tmp, log)
        os_system(cmd)

def parse_log_name(filename, parse_type):
    filename_split = filename.split('.')
    if parse_type == 'performance':
        return filename_split[-2], filename_split[-1]
    elif parse_type == 'speedup':
        return filename_split[-3], filename_split[-2], filename_split[-1]
    else:
        print "Unknow parse_type"
        exit(0)

def parse_log_content(filepath, method, parse_type):
    if parse_type == 'performance':
        parsed_filepath = filepath.replace(PERFORMANCE_LOG_DIR, PARSED_PERFORMANCE_LOG_DIR)
    elif parse_type == 'speedup':
        parsed_filepath = filepath.replace(SPEEDUP_LOG_DIR, PARSED_SPEEDUP_LOG_DIR)
    elif parse_type == 'block_nonblock':
        parsed_filepath = filepath.replace(BLOCK_NONBLOCK_LOG_DIR, PARSED_BLOCK_NONBLOCK_LOG_DIR)
    elif parse_type == 'rand_nonrand':
        parsed_filepath = filepath.replace(RAND_NONRAND_LOG_DIR, PARSED_RAND_NONRAND_LOG_DIR)
    elif parse_type == 'num_block':
        parsed_filepath = filepath.replace(NUM_BLOCK_LOG_DIR, PARSED_NUM_BLOCK_LOG_DIR)
    elif parse_type == 'nmf':
        parsed_filepath = filepath.replace(NMF_LOG_DIR, PARSED_NMF_LOG_DIR)
    else:
        print 'Error'

    with open(parsed_filepath, 'w') as out:
        if method == 'CCD' or method == 'CCD-nmf':
            for idx, line in enumerate(open(filepath)):
                if 'rmse' in line:
                    line_split = line.strip().split()
                    out.write('{0} {1}\n'.format(line_split[5], line_split[-1]))
        else:
            for idx, line in enumerate(open(filepath)):
                line_split = line.strip().split()
                if line_split[0].isdigit():
                    out.write('{0} {1}\n'.format(line_split[1], line_split[2]))

def parse_speedup_log_content(filepath, method, num_thrd):
    if method == 'HG':
        t = float(commands.getoutput('head -n {0} {1}'.format(1, filepath)).split('\n')[-1].split()[0])/float(num_thrd)
    else:
        t = commands.getoutput('tail -n {0} {1}'.format(1, filepath)).split('\n')[0].split()[0]
    return t

if __name__ == '__main__':
# Parse Log
    # Parse the log of performance
    for dirpath, dirname, filenames in os.walk(PERFORMANCE_LOG_DIR):
        for filename in filenames:
            method, dataname = parse_log_name(filename, 'performance')
            parse_log_content(os.path.join(dirpath, filename), method, 'performance')

    for dirpath, dirname, filenames in os.walk(SPEEDUP_LOG_DIR):
        for filename in filenames:
            if 'speedup' not in filename: continue
            method, dataname, nr_thread = parse_log_name(filename, 'speedup')
            parse_log_content(os.path.join(dirpath, filename), method, 'speedup')


    # Parse the log of speedup
    speedup_log_dict = collections.defaultdict(dict)
    for data in data_list:
        for method in methods_list:
            method = method.replace('_long', '')
            speedup_log_dict[data][method] = [x for x in xrange(0, 12)]
    for dirpath, dirname, filenames in os.walk(PARSED_SPEEDUP_LOG_DIR):
        for filename in filenames:
            if 'speedup' not in filename: continue
            method, dataname, num_thrd = parse_log_name(filename, 'speedup')
            method = method.replace('_long', '')
            if method not in methods_list: continue
            if dataname not in data_list: continue
            t = parse_speedup_log_content(os.path.join(dirpath, filename), method, num_thrd)
            speedup_log_dict[dataname][method][int(num_thrd) - 1] = t
    for data in speedup_log_dict.keys():
        for method in speedup_log_dict[data].keys():
            if method not in methods_list: continue
            log = os.path.join(PARSED_SPEEDUP_LOG_DIR, data, method)
            with open(log, 'w') as f:
                for idx, item in enumerate(speedup_log_dict[data][method]):
                    f.write('{0} {1}\n'.format(idx + 1, float(speedup_log_dict[data][method][0]) * 1.0/float(item)))

    # Parse the log of block v.s. non-block && rand v.s. non-rand && num_block && NMF
    for dirpath, dirname, filenames in os.walk(BLOCK_NONBLOCK_LOG_DIR):
        for filename in filenames:
            method, dataname = parse_log_name(filename, 'performance')
            parse_log_content(os.path.join(dirpath, filename), method, 'block_nonblock')
    for dirpath, dirname, filenames in os.walk(RAND_NONRAND_LOG_DIR):
        for filename in filenames:
            method, dataname = parse_log_name(filename, 'performance')
            parse_log_content(os.path.join(dirpath, filename), method, 'rand_nonrand')
    for dirpath, dirname, filenames in os.walk(NUM_BLOCK_LOG_DIR):
        for filename in filenames:
            method, dataname = parse_log_name(filename, 'performance')
            parse_log_content(os.path.join(dirpath, filename), method, 'num_block')
    for dirpath, dirname, filenames in os.walk(NMF_LOG_DIR):
        for filename in filenames:
            method, dataname = parse_log_name(filename, 'performance')
            parse_log_content(os.path.join(dirpath, filename), method, 'nmf')
