#!/usr/bin/env python
import os, sys, commands, subprocess
import collections

log_dir_list = []
mat_script_path = './plot_graph.m'
EPS_PATH = './eps/'
MATLAB_PATH = '/Applications/MATLAB_R2012a.app/bin/matlab -nosplash -nodesktop -r '

XLABEL = 'TIME (sec.)'
YLABEL = 'RMSE'

fig_list = []
fig_list.append('block_nonblock')
fig_list.append('rand_nonrand')
fig_list.append('performance')
fig_list.append('speedup')
fig_list.append('pq')
fig_list.append('nmf')

DATA_LIST = './.data_list'
data_list = []
for line in open(DATA_LIST):
    line = line.strip()
    if len(line) == 0: continue
    data_list.append(line)
os.system('mkdir -p eps')

PARSED_SPEEDUP_LOG_DIR = './log_parsed/speedup'
PARSED_PERFORMANCE_LOG_DIR = './log_parsed/performance'
PARSED_BLOCK_NONBLOCK_LOG_DIR = './log_parsed/block_nonblock'
PARSED_RAND_NONRAND_LOG_DIR = './log_parsed/rand_nonrand'
PARSED_NUM_BLOCK_LOG_DIR = './log_parsed/num_block'
PARSED_NMF_LOG_DIR = './log_parsed/nmf'

method_name_legend_map = {
    'fpsg-block':'FPSG**',
    'fpsg-nonblock':'FPSG',
    'fpsg-rand':'Random',
    'fpsg-nonrand':'Partial Random',
    'fpsg':'FPSG',
    'dsgd':'DSGD',
    'ccd':'CCD++',
    'hg':'HogWild'
}

######################## Method Color Map ##########################
method_color_map = {
    'fpsg-block':'[0, 0, 1]',
    'fpsg-nonblock':'[1, 0, 0]',
    'fpsg-rand':'[0, 1, 0]',
    'fpsg-nonrand':'[1, 0, 0]',
    'fpsg':'[1, 0, 0]',
    'dsgd':'[0, 1, 0]',
    'hg':'[0, 0, 1]',
    'ccd':'[0, 1, 1]'
}
######################## Method Color Map ##########################

######################## Line Style Map ##########################
method_line_style_map = {
    'fpsg-block':"'linestyle', '-', 'linewidth', 4",
    'fpsg-nonblock':"'linestyle', '-', 'linewidth', 4",
    'fpsg-rand':"'linestyle', '--', 'linewidth', 4",
    'fpsg-nonrand':"'linestyle', '-', 'linewidth', 4",

    'fpsg':"'linestyle', '-.', 'Marker', 's', 'linewidth', 4, 'MarkerSize', 10'",
    'dsgd':"'linestyle', '-.', 'Marker', '^', 'linewidth', 4, 'MarkerSize', 10'",
    'ccd':"'linestyle', '-.', 'Marker', 'o', 'linewidth', 4, 'MarkerSize', 10'",
    'hg':"'linestyle', '-.', 'Marker', 'd', 'linewidth', 4, 'MarkerSize', 10'"
}

######################### Line Style Map ##########################

### block v.s. non-block

PARSED_BLOCK_NONBLOCK_LOG_DIR = './log_parsed/block_nonblock'
PARSED_RAND_NONRAND_LOG_DIR = './log_parsed/rand_nonrand'

if 'block_nonblock' in fig_list:
    xlim_ylim_dict = {
        'ml':'xlim([0 25]);ylim([0.83 1])',
        'nf':'xlim([0 120]);ylim([0.91 0.985])',
        'ym':'xlim([0 750]);ylim([21.5 24.5])',
        'hw':'xlim([0 1000]);ylim([0.54 0.75])',
    }
    for data in data_list:
        script_name = 'block_nonblock_{0}'.format(data)
        with open(script_name + '.m', 'w') as out:
            block_path = os.path.join(PARSED_BLOCK_NONBLOCK_LOG_DIR, data, 'block_nonblock.FPSGD-block.{0}'.format(data))
            nonblock_path = os.path.join(PARSED_BLOCK_NONBLOCK_LOG_DIR, data, 'block_nonblock.FPSGD.{0}'.format(data))

            out.write("function {0}(varargin)\n".format(script_name))

            method_1 = 'fpsg-block'
            method_2 = 'fpsg-nonblock'

            out.write("exp_log = load('{0}');\n".format(block_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_1], method_line_style_map[method_1]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(nonblock_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_2], method_line_style_map[method_2]))
            out.write("hold on;\n")

            out.write("xlabel('Time (s)', 'fontsize', 50);\n")
            out.write("ylabel('RMSE', 'fontsize', 50);\n")
            out.write("legend('{0}', '{1}');\n".format(method_name_legend_map[method_1], method_name_legend_map[method_2]))
            out.write("set(gca, 'fontsize', 55)\n")
            out.write("{0}\n".format(xlim_ylim_dict[data]))
            out.write("set(gcf, 'Position', [0 0 1395 804], 'paperpositionmode','auto')\n")
            out.write("print('-depsc', '{0}/exp_{1}_block_nonblock.eps')\n".format(EPS_PATH, data))
            out.write("exit\n")
        cmd = '{0} {1}'.format(MATLAB_PATH, script_name)
        os.system(cmd)

if 'rand_nonrand' in fig_list:
    xlim_ylim_dict = {
        'ml':'xlim([0 160]);ylim([0.83 1])',
        'nf':'xlim([0 1600]);ylim([0.91 0.985])',
        'ym':'xlim([0 6000]);ylim([21.5 24.5])',
        'hw':'xlim([0 5000]);ylim([0.54 0.75])',
    }
    for data in data_list:
        script_name = 'rand_nonrand_{0}'.format(data)
        with open(script_name + '.m', 'w') as out:
            rand_path = os.path.join(PARSED_RAND_NONRAND_LOG_DIR, data, 'rand_nonrand.FPSGD-rand.{0}'.format(data))
            nonrand_path = os.path.join(PARSED_RAND_NONRAND_LOG_DIR, data, 'rand_nonrand.FPSGD.{0}'.format(data))

            out.write("function {0}(varargin)\n".format(script_name))

            method_1 = 'fpsg-rand'
            method_2 = 'fpsg-nonrand'

            out.write("exp_log = load('{0}');\n".format(rand_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_1], method_line_style_map[method_1]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(nonrand_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_2], method_line_style_map[method_2]))
            out.write("hold on;\n")

            out.write("xlabel('Time (s)', 'fontsize', 50);\n")
            out.write("ylabel('RMSE', 'fontsize', 50);\n")
            out.write("legend('{0}', '{1}');\n".format(method_name_legend_map[method_1], method_name_legend_map[method_2]))
            out.write("set(gca, 'fontsize', 55)\n")
            out.write("{0}\n".format(xlim_ylim_dict[data]))
            out.write("set(gcf, 'Position', [0 0 1395 804], 'paperpositionmode','auto')\n")
            out.write("print('-depsc', '{0}/exp_{1}_rand_nonrand.eps')\n".format(EPS_PATH, data))
            out.write("exit\n")
        cmd = '{0} {1}'.format(MATLAB_PATH, script_name)
        os.system(cmd)

if 'performance' in fig_list:
    xlim_ylim_dict = {
        'ml':'xlim([0 140]);ylim([0.83 0.925])',
        'nf':'xlim([0 2500]);ylim([0.91 0.965])',
        'ym':'xlim([0 3000]);ylim([21.5 24.2])',
        'hw':'xlim([0 3000]);ylim([0.54 0.81])',
    }
    for data in data_list:
        script_name = 'full_{0}'.format(data)
        with open(script_name + '.m', 'w') as out:
            fpsg_path = os.path.join(PARSED_PERFORMANCE_LOG_DIR, data, 'performance.FPSGD.{0}'.format(data))
            dsgd_path = os.path.join(PARSED_PERFORMANCE_LOG_DIR, data, 'performance.DSGD.{0}'.format(data))
            ccd_path = os.path.join(PARSED_PERFORMANCE_LOG_DIR, data, 'performance.CCD.{0}'.format(data))
            hg_path = os.path.join(PARSED_PERFORMANCE_LOG_DIR, data, 'performance.HG.{0}'.format(data))

            out.write("function {0}(varargin)\n".format(script_name))

            method_1 = 'dsgd'
            method_2 = 'hg'
            method_3 = 'ccd'
            method_4 = 'fpsg'

            out.write("exp_log = load('{0}');\n".format(dsgd_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_1], method_line_style_map[method_1]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(hg_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_2], method_line_style_map[method_2]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(ccd_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_3], method_line_style_map[method_3]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(fpsg_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_4], method_line_style_map[method_4]))
            out.write("hold on;\n")


            out.write("xlabel('Time (s)', 'fontsize', 50);\n")
            out.write("ylabel('RMSE', 'fontsize', 50);\n")
            out.write("legend('{0}', '{1}', '{2}', '{3}');\n".format(method_name_legend_map[method_1], method_name_legend_map[method_2], method_name_legend_map[method_3], method_name_legend_map[method_4]))
            out.write("set(gca, 'fontsize', 55)\n")
            out.write("{0}\n".format(xlim_ylim_dict[data]))
            out.write("set(gcf, 'Position', [0 0 1395 804], 'paperpositionmode','auto')\n")
            out.write("print('-depsc', '{0}/exp_{1}_performance.eps')\n".format(EPS_PATH, data))
            out.write("exit\n")
        cmd = '{0} {1}'.format(MATLAB_PATH, script_name)
        os.system(cmd)

if 'speedup' in fig_list:
    xlim_ylim_dict = {
        'ml':'xlim([1 12]);ylim([1 9.1])',
        'nf':'xlim([1 12]);ylim([1 9.1])',
        'ym':'xlim([1 12]);ylim([1 11.1])',
        'hw':'xlim([1 12]);ylim([1 7.1])'
    }
    for data in data_list:
        script_name = 'speedup_{0}'.format(data)
        with open(script_name + '.m', 'w') as out:
            fpsg_path = os.path.join(PARSED_SPEEDUP_LOG_DIR, data, 'FPSGD')
            dsgd_path = os.path.join(PARSED_SPEEDUP_LOG_DIR, data, 'DSGD')
            ccd_path = os.path.join(PARSED_SPEEDUP_LOG_DIR, data, 'CCD')
            hg_path = os.path.join(PARSED_SPEEDUP_LOG_DIR, data,  'HG')

            out.write("function {0}(varargin)\n".format(script_name))

            method_1 = 'dsgd'
            method_2 = 'hg'
            method_3 = 'ccd'
            method_4 = 'fpsg'

            out.write("exp_log = load('{0}');\n".format(dsgd_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_1], method_line_style_map[method_1]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(hg_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_2], method_line_style_map[method_2]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(ccd_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_3], method_line_style_map[method_3]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(fpsg_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_4], method_line_style_map[method_4]))
            out.write("hold on;\n")


            out.write("xlabel('Time (s)', 'fontsize', 50);\n")
            out.write("ylabel('RMSE', 'fontsize', 50);\n")
            out.write("legend('{0}', '{1}', '{2}', '{3}', 2);\n".format(method_name_legend_map[method_1], method_name_legend_map[method_2], method_name_legend_map[method_3], method_name_legend_map[method_4]))
            out.write("set(gca, 'fontsize', 55)\n")
            out.write("set(gca,'XTick',1:1:12);\n")
            out.write("{0}\n".format(xlim_ylim_dict[data]))
            out.write("set(gcf, 'Position', [0 0 1395 804], 'paperpositionmode','auto')\n")
            out.write("print('-depsc', '{0}/exp_{1}_speedup.eps')\n".format(EPS_PATH, data))
            out.write("exit\n")
        cmd = '{0} {1}'.format(MATLAB_PATH, script_name)
        os.system(cmd)

if 'pq' in fig_list:
    rmse_baseline_dict = {}
    rmse_baseline_dict['ml'] = 0.858
    rmse_baseline_dict['nf'] = 0.941
    rmse_baseline_dict['ym'] = 22.40

    for data in data_list:
        res_dict = collections.defaultdict(dict)
        time_res_dict = collections.defaultdict(dict)
        with open('{0}/{1}_pq.txt'.format(EPS_PATH, data), 'w') as out:
            for dirPath, dirNames, fileNames in os.walk(os.path.join(PARSED_NUM_BLOCK_LOG_DIR, data)):
                for filename in fileNames:
                    filepath = os.path.join(dirPath, filename)
                    for idx, line in enumerate(open(filepath)):
                        time, rmse = line.strip().split()
                        if float(rmse) <= float(rmse_baseline_dict[data]):
                            res_dict[int(filename.split('.')[-2])][int(filename.split('.')[-1])] = idx + 1
                            time_res_dict[int(filename.split('.')[-2])][int(filename.split('.')[-1])] = time
                            break;
            for p in sorted(res_dict.keys()):
                for q in sorted(res_dict[p].keys()):
                    out.write('{0} {1} {2} {3}\n'.format(p, q, res_dict[p][q], time_res_dict[p][q]))

if 'nmf' in fig_list:
    xlim_ylim_dict = {
        'ml':'xlim([0 15.1]);ylim([0.83 1])',
        'nf':'xlim([0 80.1]);ylim([0.92, 1.06])',
        'ym':'xlim([0 801]);ylim([22.1 25.1])',
        'hw':'xlim([0 301]);ylim([0.53 0.751])'
    }
    for data in data_list:
        script_name = 'nmf_{0}'.format(data)
        with open(script_name + '.m', 'w') as out:
            fpsg_path = os.path.join(PARSED_NMF_LOG_DIR, data, 'nmf.FPSGD-nmf.{0}'.format(data))
            ccd_path = os.path.join(PARSED_NMF_LOG_DIR, data, 'nmf.CCD-nmf.{0}'.format(data))

            out.write("function {0}(varargin)\n".format(script_name))
            method_1 = 'ccd'
            method_2 = 'fpsg'

            out.write("exp_log = load('{0}');\n".format(ccd_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_1], method_line_style_map[method_1]))
            out.write("hold on;\n")
            out.write("exp_log = load('{0}');\n".format(fpsg_path))
            out.write("plot(exp_log(:, 1), exp_log(:, 2), 'Color', {0}, {1});\n".format(method_color_map[method_2], method_line_style_map[method_2]))
            out.write("hold on;\n")

            out.write("xlabel('Time (s)', 'fontsize', 50);\n")
            out.write("ylabel('RMSE', 'fontsize', 50);\n")
            out.write("legend('{0}', '{1}', 1);\n".format(method_name_legend_map[method_1], method_name_legend_map[method_2]))
            out.write("set(gca, 'fontsize', 55)\n")
            out.write("{0}\n".format(xlim_ylim_dict[data]))
            out.write("set(gcf, 'Position', [0 0 1395 804], 'paperpositionmode','auto')\n")
            out.write("print('-depsc', '{0}/exp_{1}_nmf.eps')\n".format(EPS_PATH, data))
            out.write("exit\n")
        cmd = '{0} {1}'.format(MATLAB_PATH, script_name)
        os.system(cmd)
