#!/usr/bin/env python
import os
import sys
import time
import thread
import datetime
import threading
import subprocess
import collections
import commands

DATA_LIST = './.data_list'

data_list = []
for line in open(DATA_LIST):
    line = line.strip()
    if len(line) == 0: continue
    data_list.append(line)

SPEEDUP_LOG_DIR = './log/speedup'
PERFORMANCE_LOG_DIR = './log/performance'
BLOCK_NONBLOCK_LOG_DIR = './log/block_nonblock'
RAND_NONRAND_LOG_DIR = './log/rand_nonrand'
NUM_BLOCK_LOG_DIR = './log/num_block'
NMF_LOG_DIR = './log/nmf'

PARSED_SPEEDUP_LOG_DIR = './log_parsed/speedup'
PARSED_PERFORMANCE_LOG_DIR = './log_parsed/performance'
PARSED_BLOCK_NONBLOCK_LOG_DIR = './log_parsed/block_nonblock'
PARSED_RAND_NONRAND_LOG_DIR = './log_parsed/rand_nonrand'
PARSED_NUM_BLOCK_LOG_DIR = './log_parsed/num_block'
PARSED_NMF_LOG_DIR = './log_parsed/nmf'

dir_list = [SPEEDUP_LOG_DIR, PERFORMANCE_LOG_DIR, BLOCK_NONBLOCK_LOG_DIR, RAND_NONRAND_LOG_DIR, NUM_BLOCK_LOG_DIR, NMF_LOG_DIR, PARSED_SPEEDUP_LOG_DIR,PARSED_PERFORMANCE_LOG_DIR, PARSED_BLOCK_NONBLOCK_LOG_DIR, PARSED_RAND_NONRAND_LOG_DIR, PARSED_NUM_BLOCK_LOG_DIR, PARSED_NMF_LOG_DIR]

NUM_THRD = 12

methods_list = ['FPSGD', 'DSGD', 'HG', 'CCD']
methods_dict = {}

methods_dict['DSGD'] = './code/mf_dsgd_long/mf/mf train'
methods_dict['HG'] = './code/mf_lockfree_long/mf/mf train'
methods_dict['CCD'] = './code/libpmf-exp/omp-pmf-train'
methods_dict['FPSGD'] = './code/mf_v3_long/mf/mf train'
methods_dict['FPSGD-block'] = './code/mf_dsgd_long/mf/mf train'
methods_dict['FPSGD-rand'] = './code/mf_random_long/mf/mf train --rand-update '
methods_dict['FPSGD-pq'] =  './code/mf_v3_pq_long/mf/mf train'
methods_dict['FPSGD-nmf'] = './code/libmf_long_nmf/libmf train --nmf'
methods_dict['CCD-nmf'] =   './code/libpmf-exp/omp-pmf-train -N 1 '


data_dict = collections.defaultdict(list)
data_dict['ml'] = ['./data/ml/ml.tr.bin', './data/ml/ml.te.bin', './data/ml/libpmf']
data_dict['nf'] = ['./data/nf/nf.tr.bin', './data/nf/nf.te.bin', './data/nf/libpmf']
data_dict['ym'] = ['./data/ym/ym.tr.bin', './data/ym/ym.te.bin', './data/ym/libpmf']
data_dict['hw'] = ['./data/hw/hw.tr.bin', './data/hw/hw.te.bin', './data/hw/libpmf']
data_dict['ml-nmf'] = ['./data/ml/ml.tr.bin.nmf', './data/ml/ml.te.bin.nmf', './data/ml/libpmf']
data_dict['nf-nmf'] = ['./data/nf/nf.tr.bin.nmf', './data/nf/nf.te.bin.nmf', './data/nf/libpmf']
data_dict['ym-nmf'] = ['./data/ym/ym.tr.bin.nmf', './data/ym/ym.te.bin.nmf', './data/ym/libpmf']
# Parameters 
para_dict = {}
para_dict[('DSGD', 'ml')] = '-k 40 -t {0} -g 0.003 -p 0.05 -q 0.05 -n {1} -v {2} -gubs {1} -gibs {1} --rand-update  --rand-shuffle {3}'
para_dict[('DSGD', 'nf')] = '-k 40 -t {0} -g 0.002 -p 0.05 -q 0.05 -n {1} -v {2} -gubs {1} -gibs {1} --rand-update  --rand-shuffle {3}'
para_dict[('DSGD', 'ym')] = '-k 100 -t {0} -g 0.0001 -p 1.0 -q 1.0 -n {1} -v {2} -gubs {1} -gibs {1} --rand-update   --rand-shuffle {3}'
para_dict[('DSGD', 'hw')] = '-k 100 -t {0} -g 0.004 -p 0.01 -q 0.01 -n {1} -v {2} -gubs {1} -gibs {1} --rand-update --rand-shuffle {3}'

para_dict[('HG', 'ml')] = '-k 40 -t {0} -g 0.003 -p 0.05 -q 0.05 -n {1} -v {2} -gubs 16 -gibs 16 --rand-shuffle {3}'
para_dict[('HG', 'nf')] = '-k 40 -t {0} -g 0.002 -p 0.05 -q 0.05 -n {1} -v {2} -gubs 32 -gibs 32 --rand-shuffle {3}'
para_dict[('HG', 'ym')] = '-k 100 -t {0} -g 0.0001 -p 1.0 -q 1.0 -n {1} -v {2} -gubs 32 -gibs 32 --rand-shuffle {3}'
para_dict[('HG', 'hw')] = '-k 100 -t {0} -g 0.004 -p 0.01 -q 0.01 -n {1} -v {2} -gubs 32 -gibs 32 --rand-shuffle {3}'

para_dict[('FPSGD', 'ml')] = '-k 40 -t {0} -g 0.003 -p 0.05 -q 0.05 -n {1} -v {2} -gubs 16 -gibs 16 --rand-shuffle {3}'
para_dict[('FPSGD', 'nf')] = '-k 40 -t {0} -p 0.05 -q 0.05 -g 0.002 -n {1} -v {2} -gubs 32 -gibs 32 --rand-shuffle {3}'
para_dict[('FPSGD', 'ym')] = '-k 100 -t {0} -g 0.0001 -p 1.0 -q 1.0 -n {1} -v {2} -gubs 32 -gibs 32 --rand-shuffle {3}'
para_dict[('FPSGD', 'hw')] = '-k 100 -t {0} -g 0.004 -p 0.01 -q 0.01 -n {1} -v {2} -gubs 32 -gibs 32 --rand-shuffle {3}'
para_dict[('FPSGD-block', 'ml')] = '-k 40 -t {0} -g 0.003 -p 0.05 -q 0.05 -n {1} -v {2} -gubs {1} -gibs {1} --rand-shuffle {3}'
para_dict[('FPSGD-block', 'nf')] = '-k 40 -t {0} -p 0.05 -q 0.05 -g 0.002 -n {1} -v {2} -gubs {1} -gibs {1} --rand-shuffle {3}'
para_dict[('FPSGD-block', 'ym')] = '-k 100 -t {0} -g 0.0001 -p 1.0 -q 1.0 -n {1} -v {2} -gubs {1} -gibs {1} --rand-shuffle {3}'
para_dict[('FPSGD-block', 'hw')] = '-k 100 -t {0} -g 0.004 -p 0.01 -q 0.01 -n {1} -v {2} -gubs {1} -gibs {1} --rand-shuffle {3}'

para_dict[('FPSGD-pq', 'ml')] = '-k 40 -t {0} -g 0.003 -p 0.05 -q 0.05 -n {1} -v {2} -gubs {4} -gibs {5} --rand-shuffle {3}'
para_dict[('FPSGD-pq', 'nf')] = '-k 40 -t {0} -p 0.05 -q 0.05 -g 0.002 -n {1} -v {2} -gubs {4} -gibs {5} --rand-shuffle {3}'
para_dict[('FPSGD-pq', 'ym')] = '-k 100 -t {0} -g 0.0001 -p 1.0 -q 1.0 -n {1} -v {2} -gubs {4} -gibs {5} --rand-shuffle {3}'
para_dict[('FPSGD-pq', 'hw')] = '-k 100 -t {0} -g 0.004 -p 0.01 -q 0.01 -n {1} -v {2} -gubs {4} -gibs {5} --rand-shuffle {3}'

para_dict[('FPSGD-nmf', 'ml')] = '-k 40  -t {0} -g 0.01  -p 0.05 -q 0.05 -s {1} -v {2} -blk 16x16 --rand-shuffle {3}'
para_dict[('FPSGD-nmf', 'nf')] = '-k 40  -t {0} -p 0.05   -q 0.05 -g 0.005 -s {1} -v {2} -blk 32x32  --rand-shuffle {3}'
para_dict[('FPSGD-nmf', 'ym')] = '-k 100 -t {0} -g 0.0001 -p 1.0  -q 1.0  -s {1} -v {2} -blk 32x32 --rand-shuffle {3}'
para_dict[('FPSGD-nmf', 'hw')] = '-k 100 -t {0} -g 0.004  -p 0.01 -q 0.01 -s {1} -v {2} -blk 32x32  --rand-shuffle {3}'

para_dict[('CCD', 'ml')] = '-k 40 -t {0} -n {1} -l 0.05 -q 1 -p 1 {2}'
para_dict[('CCD', 'nf')] = '-k 40 -t {0} -n {1} -l 0.05 -q 1 -p 1 {2}'
para_dict[('CCD', 'ym')] = '-k 100 -t {0} -n {1} -l 1.0 -q 1 -p 1 {2}'
para_dict[('CCD', 'hw')] = '-k 100 -t {0} -n {1} -l 0.01 -q 1 -p 1 {2}'

iter_dict = {}
iter_dict[('DSGD', 'ml')] = 200
iter_dict[('DSGD', 'nf')] = 250
iter_dict[('DSGD', 'ym')] = 200
iter_dict[('DSGD', 'hw')] = 50
iter_dict[('HG', 'ml')] = 60
iter_dict[('HG', 'nf')] = 200
iter_dict[('HG', 'ym')] = 70
iter_dict[('HG', 'hw')] = 15
iter_dict[('CCD', 'ml')] = 50
iter_dict[('CCD', 'nf')] = 50
iter_dict[('CCD', 'ym')] = 30
iter_dict[('CCD', 'hw')] = 50
iter_dict[('FPSGD', 'ml')] = 200
iter_dict[('FPSGD', 'nf')] = 250
iter_dict[('FPSGD', 'ym')] = 250
iter_dict[('FPSGD', 'hw')] = 50

def os_system(cmd):
    print cmd
    os.system(cmd)

def subp_run(log_tmp, log, cmd):
        print cmd
        f = open(log_tmp, 'w')
        subp = subprocess.Popen(cmd.split(), shell=False, stdout=f)
        subp.wait()
        f.close()
        
        cmd = 'mv {0} {1}'.format(log_tmp, log)
        os_system(cmd)

def eval_performance(method, data):
    log_tmp = os.path.join(PERFORMANCE_LOG_DIR, data, '{0}.{1}.{2}.tmp'.format('performance', method, data))
    log = os.path.join(PERFORMANCE_LOG_DIR, data, '{0}.{1}.{2}'.format('performance', method, data))

    if method == 'CCD':
        cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(iter_dict[(method, data)], NUM_THRD, data_dict[data][2]))
    else:
        cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(iter_dict[(method, data)], NUM_THRD, data_dict[data][1], data_dict[data][0]))
    subp_run(log_tmp, log, cmd)

def eval_speedup(method, data):
    for num_thrd in xrange(0, 12):
        num_thrd += 1

        if method == 'CCD':
            cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(5, num_thrd, data_dict[data][2]))
        else:
            cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(15, num_thrd, data_dict[data][1], data_dict[data][0]))

        log_tmp = os.path.join(SPEEDUP_LOG_DIR, data, '{0}.{1}.{2}.{3}.tmp'.format('speedup', method, data, num_thrd))
        log = os.path.join(SPEEDUP_LOG_DIR, data, '{0}.{1}.{2}.{3}'.format('speedup', method, data, num_thrd))
        subp_run(log_tmp, log, cmd)


def eval_block_non_block(method, data):
    log_tmp = os.path.join(BLOCK_NONBLOCK_LOG_DIR, data, '{0}.{1}.{2}.tmp'.format('block_nonblock', method, data))
    log = os.path.join(BLOCK_NONBLOCK_LOG_DIR, data, '{0}.{1}.{2}'.format('block_nonblock', method, data))
    cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(iter_dict[('FPSGD', data)], NUM_THRD, data_dict[data][1], data_dict[data][0]))
    subp_run(log_tmp, log, cmd)

def eval_rand_non_rand(method, data):
    log_tmp = os.path.join(RAND_NONRAND_LOG_DIR, data, '{0}.{1}.{2}.tmp'.format('rand_nonrand', method, data))
    log = os.path.join(RAND_NONRAND_LOG_DIR, data, '{0}.{1}.{2}'.format('rand_nonrand', method, data))
    cmd = '{0} {1}'.format(methods_dict[method], para_dict[('FPSGD', data)].format(iter_dict[('FPSGD', data)], NUM_THRD, data_dict[data][1], data_dict[data][0]))
    subp_run(log_tmp, log, cmd)

def eval_num_block(method, data):
    block_list = [16, 32, 64, 128, 256]
    for num_block_p in block_list:
        for num_block_q in block_list:
            log_tmp = os.path.join(NUM_BLOCK_LOG_DIR, data, '{0}.{1}.{2}.{3}.{4}.tmp'.format('num_block', method, data, num_block_p, num_block_q))
            log = os.path.join(NUM_BLOCK_LOG_DIR, data, '{0}.{1}.{2}.{3}.{4}'.format('num_block', method, data, num_block_p, num_block_q))
            cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(50, NUM_THRD, data_dict[data][1], data_dict[data][0], num_block_p, num_block_q))
            subp_run(log_tmp, log, cmd)

def eval_nmf(method, data):
    log_tmp = os.path.join(NMF_LOG_DIR, data, '{0}.{1}.{2}.tmp'.format('nmf', method, data))
    log = os.path.join(NMF_LOG_DIR, data, '{0}.{1}.{2}'.format('nmf', method, data))
    if method  == 'CCD-nmf':
        cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method.split('-')[0], data)].format(iter_dict[(method.split('-')[0], data)], NUM_THRD, data_dict[data+'-nmf'][-1]))
    else:
        cmd = '{0} {1}'.format(methods_dict[method], para_dict[(method, data)].format(iter_dict[(method.split('-')[0], data)], NUM_THRD, data_dict[data+'-nmf'][1], data_dict[data+'-nmf'][0]))
    subp_run(log_tmp, log, cmd)

def parse_log_name(filename, parse_type):
    filename_split = filename.split('.')
    if parse_type == 'performance':
        return filename_split[-2], filename_split[-1]
    elif parse_type == 'speedup':
        return filename_split[-3], filename_split[-2], filename_split[-1]
    else:
        print "Unknow parse_type"
        exit(0)

def parse_log_content(filepath, method, parse_type):
    if parse_type == 'performance':
        parsed_filepath = filepath.replace(PERFORMANCE_LOG_DIR, PARSED_PERFORMANCE_LOG_DIR)
    elif parse_type == 'speedup':
        parsed_filepath = filepath.replace(SPEEDUP_LOG_DIR, PARSED_SPEEDUP_LOG_DIR)
    elif parse_type == 'block_nonblock':
        parsed_filepath = filepath.replace(BLOCK_NONBLOCK_LOG_DIR, PARSED_BLOCK_NONBLOCK_LOG_DIR)
    elif parse_type == 'rand_nonrand':
        parsed_filepath = filepath.replace(RAND_NONRAND_LOG_DIR, PARSED_RAND_NONRAND_LOG_DIR)
    elif parse_type == 'num_block':
        parsed_filepath = filepath.replace(NUM_BLOCK_LOG_DIR, PARSED_NUM_BLOCK_LOG_DIR)
    elif parse_type == 'nmf':
        parsed_filepath = filepath.replace(NMF_LOG_DIR, PARSED_NMF_LOG_DIR)
    else:
        print 'Error'

    with open(parsed_filepath, 'w') as out:
        if method == 'CCD' or method == 'CCD-nmf':
            for idx, line in enumerate(open(filepath)):
                if 'rmse' in line:
                    line_split = line.strip().split()
                    out.write('{0} {1}\n'.format(line_split[5], line_split[-1]))
        else:
            for idx, line in enumerate(open(filepath)):
                line_split = line.strip().split()
                if line_split[0].isdigit():
                    out.write('{0} {1}\n'.format(line_split[1], line_split[2]))

def parse_speedup_log_content(filepath, method, num_thrd):
    if method == 'HG':
        t = commands.getoutput('head -n {0} {1}'.format(12 - int(num_thrd) + 1, filepath)).split('\n')[-1].split()[0]
    else:
        t = commands.getoutput('tail -n {0} {1}'.format(1, filepath)).split('\n')[0].split()[0]
    return t

if __name__ == '__main__':

    print 'Create the log dir'

    for data in data_list:
        for item in dir_list:
            os.system('mkdir -p {0}/{1}'.format(item, data))

# Test performance
    print '--------------------------------'
    print 'Begin to test the performance'


    for method in methods_list:
        print 'Begin to test the performance'
        for data in data_list:
            if (method, data) not in para_dict: continue
            print '###################################'
            cmd = 'mkdir -p {0}'.format(os.path.join(PERFORMANCE_LOG_DIR, data))
            os_system(cmd)
            print method, data
            eval_performance(method, data)
# Test Speedup
    print '--------------------------------'
    print 'Begin to test the speedup'
    for method in methods_list:
        for data in data_list:
            if (method, data) not in para_dict: continue
            print '###################################'
            cmd = 'mkdir -p {0}'.format(os.path.join(SPEEDUP_LOG_DIR, data))
            os_system(cmd)
            print method, data
            eval_speedup(method, data)

# Test Block v.s. Non-Block
    print '--------------------------------'
    print 'Begin to test the speedup'
    for method in ['FPSGD', 'FPSGD-block']:
        for data in data_list:
            if (method, data) not in para_dict: continue
            print '###################################'
            cmd = 'mkdir -p {0}'.format(os.path.join(SPEEDUP_LOG_DIR, data))
            os_system(cmd)
            eval_block_non_block(method, data)

# Test Rand v.s. Non-Rand
    print '--------------------------------'
    print 'Begin to test the speedup'
    for method in ['FPSGD', 'FPSGD-rand']:
        for data in data_list:
#            if (method, data) not in para_dict: continue
            print '###################################'
            cmd = 'mkdir -p {0}'.format(os.path.join(SPEEDUP_LOG_DIR, data))
            os_system(cmd)
            eval_rand_non_rand(method, data)

# Test number of blocks
    for method in ['FPSGD-pq']:
        for data in data_list:
            print '###################################'
            cmd = 'mkdir -p {0}'.format(os.path.join(NUM_BLOCK_LOG_DIR, data))
            os_system(cmd)
            eval_num_block(method, data)
# Test NMF
    for method in ['FPSGD-nmf', 'CCD-nmf']:
        for data in data_list:
            print '###################################'
#            cmd = 'mkdir -p {0}'.format(os.path.join(NUM_BLOCK_LOG_DIR, data))
#            os_system(cmd)
            eval_nmf(method, data)
